<#
.SYNOPSIS
    Internet Optimizer by RAIYAN
    Sets Cloudflare DNS for both IPv4 and IPv6 and applies advanced internet optimization settings.
#>

# Define Cloudflare DNS servers
$cloudflareDNS1 = "1.1.1.1"
$cloudflareDNS2 = "1.0.0.1"
$cloudflareDNS1_v6 = "2606:4700:4700::1111"
$cloudflareDNS2_v6 = "2606:4700:4700::1001"

# Get active network adapters
$networkAdapters = Get-NetAdapter | Where-Object { $_.Status -eq "Up" }

# Function to set DNS for an adapter
function Set-DnsForAdapter {
    param (
        [Parameter(Mandatory = $true)]
        [string]$AdapterName,
        
        [Parameter(Mandatory = $true)]
        [string]$PrimaryDNS,
        
        [Parameter(Mandatory = $true)]
        [string]$SecondaryDNS,
        
        [Parameter(Mandatory = $false)]
        [string]$PrimaryDNS_v6,
        
        [Parameter(Mandatory = $false)]
        [string]$SecondaryDNS_v6
    )

    # Set IPv4 DNS servers for the network adapter
    Set-DnsClientServerAddress -InterfaceAlias $AdapterName -ServerAddresses ($PrimaryDNS, $SecondaryDNS)

    # Set IPv6 DNS servers if provided
    if ($PrimaryDNS_v6 -and $SecondaryDNS_v6) {
        Set-DnsClientServerAddress -InterfaceAlias $AdapterName -ServerAddresses ($PrimaryDNS_v6, $SecondaryDNS_v6) -AddressFamily IPv6
    }
}

# Function to optimize internet settings
function Optimize-InternetSettings {
    Write-Output "Applying internet optimization settings..."

    # Disable auto-tuning (can improve performance in some cases)
    Write-Output "Disabling auto-tuning..."
    netsh int tcp set global autotuninglevel=disabled

    # Enable Receive Window Auto-Tuning Level
    Write-Output "Enabling Receive Window Auto-Tuning Level..."
    netsh int tcp set global autotuninglevel=normal

    # Set TCP time wait delay to default
    Write-Output "Setting TCP time wait delay to default..."
    netsh int ipv4 set dynamicport tcp start=49152 num=16384

    # Enable QoS Packet Scheduler and set reserve bandwidth to 0%
    Write-Output "Enabling QoS Packet Scheduler and setting reserve bandwidth to 0%..."
    $qosPath = "HKLM:\SOFTWARE\Policies\Microsoft\Windows\Psched"
    if (-Not (Test-Path $qosPath)) {
        New-Item -Path $qosPath -Force | Out-Null
    }
    New-ItemProperty -Path $qosPath -Name "NonBestEffortLimit" -Value 0 -PropertyType DWord -Force | Out-Null

    # Disable Power Saving on Network Adapters using PowerCfg
    Write-Output "Disabling power saving on network adapters..."
    $networkAdapters | ForEach-Object {
        $adapterName = $_.Name
        Get-NetAdapterPowerManagement -Name $adapterName | Disable-NetAdapterPowerManagement
        powercfg -devicequery wake_armed | ForEach-Object { powercfg -devicedisablewake $_ }
    }

    # Additional Settings
    Write-Output "Applying additional network settings..."
    
    # Enable ECN Capability (Explicit Congestion Notification)
    Write-Output "Enabling ECN Capability..."
    netsh int tcp set global ecncapability=enabled

    # Enable RSS (Receive Side Scaling)
    Write-Output "Enabling RSS..."
    netsh int tcp set global rss=enabled

    Write-Output "Internet optimization settings applied successfully."
}

# Loop through each active adapter and set DNS
foreach ($adapter in $networkAdapters) {
    try {
        Write-Output "Setting Cloudflare DNS for adapter: $($adapter.Name)"
        Set-DnsForAdapter -AdapterName $adapter.Name `
                          -PrimaryDNS $cloudflareDNS1 `
                          -SecondaryDNS $cloudflareDNS2 `
                          -PrimaryDNS_v6 $cloudflareDNS1_v6 `
                          -SecondaryDNS_v6 $cloudflareDNS2_v6
        Write-Output "Successfully set Cloudflare DNS for adapter: $($adapter.Name)"
    } catch {
        Write-Output "Failed to set Cloudflare DNS for adapter: $($adapter.Name). Error: $_"
    }
}

# Apply internet optimization settings
Optimize-InternetSettings

# Verify DNS settings
Write-Output "Current DNS settings for active adapters:"
Get-DnsClientServerAddress | Where-Object { $_.InterfaceAlias -in $networkAdapters.Name } | Select-Object InterfaceAlias, ServerAddresses

Write-Output "Internet optimization completed by Internet_Optimizer_Scirpt by RAIYAN."
