<#
.SYNOPSIS
    System Optimizer by RAIYAN
    Enhances system performance and privacy by setting a high-performance power plan, disabling power throttling, 
    disabling unnecessary logging and services, and applying additional optimizations.
#>

# Function to set the power plan to High Performance
function Set-HighPerformancePowerPlan {
    Write-Output "Setting power plan to High Performance..."
    powercfg -setactive SCHEME_MIN
    Write-Output "Power plan set to High Performance."
}

# Function to disable power throttling
function Disable-PowerThrottling {
    Write-Output "Disabling power throttling..."
    $keyPath = "HKLM:\SYSTEM\CurrentControlSet\Control\Power"
    if (Test-Path $keyPath) {
        New-ItemProperty -Path $keyPath -Name "PowerThrottlingOff" -Value 1 -PropertyType DWord -Force | Out-Null
        Write-Output "Power throttling disabled."
    } else {
        Write-Output "Power throttling path not found."
    }
}

# Function to disable unnecessary logging and reporting
function Disable-LoggingAndReporting {
    Write-Output "Disabling unnecessary logging and reporting..."

    $servicesToDisable = @(
        "WerSvc",  # Windows Error Reporting Service
        "DiagTrack" # Connected User Experiences and Telemetry
    )

    foreach ($service in $servicesToDisable) {
        if (Get-Service -Name $service -ErrorAction SilentlyContinue) {
            Set-Service -Name $service -StartupType Disabled
            Stop-Service -Name $service -Force
            Write-Output "$service disabled."
        } else {
            Write-Output "$service not found."
        }
    }

    Write-Output "Unnecessary logging and reporting disabled."
}

# Function to disable unnecessary services
function Disable-UnnecessaryServices {
    Write-Output "Disabling unnecessary services..."

    $servicesToDisable = @(
        "RemoteRegistry",
        "Fax",
        "dmwappushservice",  # Device Management Wireless Application Protocol (WAP) Push message Routing Service
        "MapsBroker",        # Downloaded Maps Manager
        "DiagSvc",           # Diagnostic Policy Service
        "RetailDemo"         # Retail Demo Service
    )

    foreach ($service in $servicesToDisable) {
        if (Get-Service -Name $service -ErrorAction SilentlyContinue) {
            Set-Service -Name $service -StartupType Disabled
            Stop-Service -Name $service -Force
            Write-Output "$service disabled."
        } else {
            Write-Output "$service not found."
        }
    }

    Write-Output "Unnecessary services disabled."
}

# Function to set optional services to Manual mode
function Set-OptionalServicesToManual {
    Write-Output "Setting less critical services to Manual mode (optional)..."

    $servicesToManual = @(
        "PrintNotify",
        "TabletInputService",
        "CscService",
        "AppReadiness",
        "Netlogon",
        "WMPNetworkSvc"
    )

    foreach ($service in $servicesToManual) {
        if (Get-Service -Name $service -ErrorAction SilentlyContinue) {
            Set-Service -Name $service -StartupType Manual
            Write-Output "$service set to Manual."
        } else {
            Write-Output "$service not found."
        }
    }

    Write-Output "Less critical services set to Manual mode."
}

# Function to apply additional privacy and performance enhancements
function Apply-AdditionalEnhancements {
    Write-Output "Applying additional privacy and performance enhancements..."

    $registryChanges = @(
        @{Path="HKLM:\SOFTWARE\Policies\Microsoft\Windows\Windows Search"; Name="AllowCortana"; Value=0},
        @{Path="HKLM:\SOFTWARE\Policies\Microsoft\Windows\DataCollection"; Name="AllowTelemetry"; Value=0},
        @{Path="HKLM:\SOFTWARE\Policies\Microsoft\Windows\CloudContent"; Name="DisableWindowsConsumerFeatures"; Value=1},
        @{Path="HKLM:\SOFTWARE\Policies\Microsoft\SQMClient\Windows"; Name="CEIPEnable"; Value=0},
        @{Path="HKCU:\Software\Microsoft\Windows\CurrentVersion\PushNotifications"; Name="ToastEnabled"; Value=0},
        @{Path="HKLM:\SOFTWARE\Microsoft\Windows\CurrentVersion\DeliveryOptimization\Config"; Name="DODownloadMode"; Value=0},
        @{Path="HKLM:\SOFTWARE\Policies\Microsoft\Windows\CloudContent"; Name="DisableSoftLanding"; Value=1}
    )

    foreach ($change in $registryChanges) {
        if (Test-Path $change.Path) {
            Set-ItemProperty -Path $change.Path -Name $change.Name -Value $change.Value -Force | Out-Null
            Write-Output "Set $($change.Name) to $($change.Value) in $($change.Path)"
        } else {
            Write-Output "Path $($change.Path) not found."
        }
    }

    Write-Output "Additional privacy and performance enhancements applied."
}

# Additional function to disable Windows telemetry
function Disable-WindowsTelemetry {
    Write-Output "Disabling Windows telemetry..."

    $servicesToDisable = @(
        "DiagTrack",
        "dmwappushservice",
        "PcaSvc",  # Program Compatibility Assistant Service
        "WMPNetworkSvc"  # Windows Media Player Network Sharing Service
    )

    foreach ($service in $servicesToDisable) {
        if (Get-Service -Name $service -ErrorAction SilentlyContinue) {
            Set-Service -Name $service -StartupType Disabled
            Stop-Service -Name $service -Force
            Write-Output "$service disabled."
        } else {
            Write-Output "$service not found."
        }
    }

    Write-Output "Windows telemetry disabled."
}

# Apply the optimizations
Set-HighPerformancePowerPlan
Disable-PowerThrottling
Disable-LoggingAndReporting
Disable-UnnecessaryServices
Set-OptionalServicesToManual
Apply-AdditionalEnhancements
Disable-WindowsTelemetry

Write-Output "System optimization completed by System_Optimizer_Script by RAIYAN."